package com.jpsoft.picc.modules.auth.controller;

import com.github.pagehelper.Page;
import com.jpsoft.picc.modules.base.entity.Company;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinition;
import com.jpsoft.picc.modules.base.entity.InsuranceDefinitionLimit;
import com.jpsoft.picc.modules.base.entity.InsuranceJobs;
import com.jpsoft.picc.modules.base.service.CompanyService;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionLimitService;
import com.jpsoft.picc.modules.base.service.InsuranceDefinitionService;
import com.jpsoft.picc.modules.base.service.InsuranceJobsService;
import com.jpsoft.picc.modules.business.entity.InsuranceApplication;
import com.jpsoft.picc.modules.business.service.InsuranceApplicationService;
import com.jpsoft.picc.modules.common.constant.PolicyStatus;
import com.jpsoft.picc.modules.common.dto.MessageResult;
import com.jpsoft.picc.modules.common.dto.Sort;
import com.jpsoft.picc.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.jasig.cas.client.authentication.AttributePrincipal;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Api(description="投保信息")
@RestController
@RequestMapping("/auth/insuranceApplication")
public class InsuranceApplicationController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private InsuranceApplicationService insuranceApplicationService;

    @Autowired
    private InsuranceDefinitionService insuranceDefinitionService;

    @Autowired
    private InsuranceDefinitionLimitService insuranceDefinitionLimitService;

    @Autowired
    private InsuranceJobsService insuranceJobsService;

    @Autowired
    private CompanyService companyService;

    @ApiOperation(value="投保单列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    @ApiImplicitParams({
//            @ApiImplicitParam(name = "status",value = "状态:草稿10/待初审20/待复审30/待缴费40/待制单50/待出单60/已出单70/已过期100", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Map> pageList(
//            @RequestParam(value="status",defaultValue="") String status,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(company.getId())) {
            searchParams.put("companyId",company.getId());
        }

        Page<InsuranceApplication> page = insuranceApplicationService.pageSearch(searchParams,pageIndex,pageSize,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="获取投保信息")
    @RequestMapping(value = "insureDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Map<String,Object>> insureDetail(@RequestParam(value="id",defaultValue="") String id,
                                                          HttpServletRequest request){
        MessageResult<Map<String,Object>> msgResult = new MessageResult<>();

        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());
        if(company == null){
            msgResult.setResult(false);
            msgResult.setMessage("请先完善企业信息！");

            return msgResult;
        }

        Map<String,Object> objectMap = new HashMap<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);

            if (insuranceApplication != null) {
                insuranceApplication = insuranceApplicationService.get(id);
            } else {
                insuranceApplication = new InsuranceApplication();
            }

            objectMap.put("insuranceApplication",insuranceApplication);
            objectMap.put("company",company);

            msgResult.setData(objectMap);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取承保信息")
    @RequestMapping(value = "acceptDetail",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<Map<String,Object>> acceptDetail(@RequestParam(value="id",defaultValue="") String id){
        MessageResult<Map<String,Object>> msgResult = new MessageResult<>();
        Map<String,Object> objectMap = new HashMap<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);

            InsuranceDefinition insuranceDefinition = insuranceDefinitionService.get(insuranceApplication.getDefinitionId());

            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("definitionId",insuranceApplication.getDefinitionId());
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("id_","asc"));

            Page<InsuranceDefinitionLimit> insuranceDefinitionLimitPage = insuranceDefinitionLimitService.pageSearch(searchParams,1,100,sortList);
            insuranceDefinition.setLimits(insuranceDefinitionLimitPage.getResult());
            Page<InsuranceJobs> insuranceJobsPage = insuranceJobsService.pageSearch(searchParams,1,100,sortList);

            objectMap.put("insuranceDefinition",insuranceDefinition);
            objectMap.put("insuranceJobs",insuranceJobsPage.getResult());

            msgResult.setData(objectMap);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="提交保单")
    @RequestMapping(value = "submit",method = RequestMethod.POST)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id",value = "编号", required = true, paramType = "form",dataType = "String")
    })
    public MessageResult<InsuranceApplication> submit(@RequestParam(value="id",defaultValue="") String id,
                                                      HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();

        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            InsuranceApplication insuranceApplication = insuranceApplicationService.get(id);

            insuranceApplication.setStatus(String.valueOf(PolicyStatus.PendingTrial.getValue()));
            insuranceApplication.setUpdateBy(principal.getName());
            insuranceApplication.setUpdateTime(new Date());
            int affectCount = insuranceApplicationService.update(insuranceApplication);

            msgResult.setResult(true);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="保存草稿")
    @RequestMapping(value = "saveDraft",method = RequestMethod.POST)
    public MessageResult<InsuranceApplication> saveDraft(@ModelAttribute InsuranceApplication insuranceApplication,
                                                        HttpServletRequest request){
        AttributePrincipal principal = (AttributePrincipal) request.getUserPrincipal();
        Company company = companyService.findByCreateBy(principal.getName());

        MessageResult<InsuranceApplication> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;
            if(insuranceApplication.getId() == null){
                insuranceApplication.setId(UUID.randomUUID().toString());
                insuranceApplication.setCompanyId(company.getId());
                insuranceApplication.setStatus(String.valueOf(PolicyStatus.Draft));
                insuranceApplication.setDelFlag(false);
                insuranceApplication.setCreateBy(principal.getName());
                insuranceApplication.setCreateTime(new Date());

                affectCount = insuranceApplicationService.insert(insuranceApplication);
            }else {
                insuranceApplication.setUpdateBy(principal.getName());
                insuranceApplication.setUpdateTime(new Date());

                affectCount = insuranceApplicationService.update(insuranceApplication);
            }

            if(affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(insuranceApplication);
            }else {
                msgResult.setResult(false);
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
